import java.util.ArrayList;
import java.util.List;

import jcsp.lang.*;

/**
   This class was adapted from the version found on
   http://www.cs.kent.ac.uk/projects/ofa/java-threads/0.html.
   
   This is the main class for this solution of the
   Greedy Philosopher Problem using the J-CSP library.
   It consists of a college with philosophers, a canteen, and a chef.
   The first philosopher will be "greedy" and will try to rush into
   the canteen to get a chicken before the rest.
   
   @author taustin
*/
public class College
{
   //The Philosophers.  The first will be the greedy Philosopher.
   private List<Philosopher> philosophers;
   
   //The Chef, who will cook 4 chickens at a time.
   private Chef chef;
   
   //The college canteen, where the philosophers eat.
   // NOTE THAT THIS IS USED TO REPLACE THE Canteen CLASS
   // FROM THE ORIGINAL SOLUTION!
   private Channel canteen;
   
   /**
      Constructor.
      
      @param numPhilosophers  Number of Philosophers in the College.
    */
   public College(int numPhilosophers)
   {
      canteen = new One2AnyChannel();
      chef = new Chef (canteen);
      philosophers = new ArrayList<Philosopher>();
      for (int i=0; i<numPhilosophers; i++)
      {
         //only Philosopher 0 is greedy.
         philosophers.add(new Philosopher(i, canteen, i==0));
      }
   }
   
   /**
      Starts the program by starting the threads for the
      Chef and each of the Philosophers.
    */
   public void start()
   {
      //We have one process for each philosopher, plus one for the chef.
      CSProcess[] processes = new CSProcess[philosophers.size() + 1];
      processes[0] = chef;
      for (int i=1; i<processes.length; i++)
         processes[i] = philosophers.get(i-1);
      
      Parallel p = new Parallel(processes);
      p.run();
   }
   
   /**
      Runs the demo.
    */
   public static void main (String argv[])
   {
      College college = new College(5);
      college.start();
   }
}



